#!/usr/bin/env python
#
# PkgTrackInst.InstallSession - Tracks installation log
#
# ----------------------------------------------------------------------------
# "THE BEER-WARE LICENSE" (Revision 42, (c) Poul-Henning Kamp):
# Hye-Shik Chang <perky@FreeBSD.org> wrote this file.  As long as you retain
# this notice you can do whatever you want with this stuff. If we meet some
# day, and you think this stuff is worth it, you can buy me a beer in return.
#
# Hye-Shik Chang
# ----------------------------------------------------------------------------
# $Id: InstallSession.py 507 2005-05-12 23:44:01Z perky $

__pychecker__ = 'no-argsused no-isliteral'

import os, sys
from PkgTrackInst import options
from PkgTrackInst.cmd_support import errx
from PkgTrackInst.Trackers import \
        InstFile, InstDirectory, InstSymbolicLink, TrackRoot

DELIM = '\x01'

class InstallSession(object):

    def feed(self, line):
        args = line.split(DELIM)

        pid, func, errorcode = args[:3]
        pid = int(pid)
        if func == 'newproc':
            return

        errorcode = int(errorcode)
        funcargs = args[3:]
        handler_name = 'hdl_' + func

        if not hasattr(self, handler_name):
            errx("function `%s' isn't supported in %s." % (func, sys.argv[0]))
        elif not errorcode:
            getattr(self, handler_name)(*funcargs)

    def new_file(self, path):
        TrackRoot[path] = InstFile(created=True)

    def new_directory(self, path):
        TrackRoot[path] = InstDirectory(created=True)

    def new_symlink(self, src, dst):
        TrackRoot[dst] = InstSymbolicLink(src, created=True)

    def move(self, src, dst):
        dirname, basename = os.path.split(src)
        TrackRoot[dirname].move(basename, dst)

    def delete(self, path):
        del TrackRoot[path]

    # -*- libc log handlers -*-

    hdl_creat = new_file
    hdl_fopen = new_file

    def hdl_link(self, src, dst):
        # copy filename part from src if dst is a directory
        if os.path.isdir(dst):
            namepart = os.path.split(src)[-1]
            dst = os.path.join(dst, namepart)
        self.new_file(dst)

    hdl_mkdir = new_directory
    hdl_open = new_file
    hdl_rename = move
    hdl_rmdir = delete
    hdl_symlink = new_symlink

    def hdl_truncate(self, ret, path, length):
        self.new_file(path)

    hdl_unlink = delete
