#!/usr/bin/env python
#
# PkgTrackInst.TerminalSlider - Shows rectangular filler progress bar.
#
# ----------------------------------------------------------------------------
# "THE BEER-WARE LICENSE" (Revision 42, (c) Poul-Henning Kamp):
# Hye-Shik Chang <perky@FreeBSD.org> wrote this file.  As long as you retain
# this notice you can do whatever you want with this stuff. If we meet some
# day, and you think this stuff is worth it, you can buy me a beer in return.
# ----------------------------------------------------------------------------
#
# $Id: TerminalSlider.py 476 2005-05-01 14:38:12Z perky $

from __future__ import division
import os, sys, re
import time

def dotint(v):
    dotstr = str(v)
    left = dotstr[:len(dotstr)%3]
    right = re.findall('...', dotstr[len(dotstr)%3:])
    if left:
        right.insert(0, left)
    return ','.join(right)

class TerminalSlider:
    def __init__(self, maxpos, initpos):
        self.barfill = -1
        self.maxpos = maxpos
        self.initpos = initpos
        self.sessmax = maxpos - initpos
        try:
            import fcntl, termios, struct
            s = struct.pack("HHHH", 0, 0, 0, 0)
            self.cols = struct.unpack("HHHH", fcntl.ioctl(sys.stdout.fileno(),
                                termios.TIOCGWINSZ, s))[1]
        except:
            self.cols = 78

        # 4(percent) + 2([]) + 4(spaces) + 2(endmark)
        self.barsize = self.cols - 12
        self.barsize -= len(dotint(maxpos)) # done value
        self.barfmt = '\r%%-4s [%%s>%%s] %%-%ds ' % len(dotint(maxpos))
        self.barsize -= 23
        self.st = time.time()
        self.update(0)
    
    def update(self, value, *ext):
        if self.maxpos > 0:
            newfill = (value + self.initpos) * self.barsize // self.maxpos
        else:
            newfill = self.barsize
        self.barfill = newfill
        sys.stdout.write(self.barfmt %
            (str(newfill * 100 // self.barsize)+'%',
             '='*(newfill - 1), ' '*(self.barsize-newfill), dotint(value)))

        if value * 30 >= self.sessmax: # over 3%
            elapsed = time.time() - self.st
            if value > 0:
                estimated = int(self.sessmax / value * elapsed - elapsed)
            else:
                estimated = self.sessmax
            sys.stdout.write("   ETA %02d:%02d" %
                             (estimated // 60, estimated % 60))
        sys.stdout.flush()
    
    def end(self):
        print


if __name__ == '__main__':
    f = TerminalSlider(100, 0)
    for i in range(100):
        time.sleep(0.2)
        f.update(i)
