#!/usr/bin/env python
#
# PkgTrackInst.genplist_main - Tracks installation log and generates pkg-plist.
#
# ----------------------------------------------------------------------------
# "THE BEER-WARE LICENSE" (Revision 42, (c) Poul-Henning Kamp):
# Hye-Shik Chang <perky@FreeBSD.org> wrote this file.  As long as you retain
# this notice you can do whatever you want with this stuff. If we meet some
# day, and you think this stuff is worth it, you can buy me a beer in return.
#
# Hye-Shik Chang
# ----------------------------------------------------------------------------
# $Id: genplist_main.py 525 2005-05-15 11:20:54Z perky $

__pychecker__ = 'no-argsused no-isliteral'

from PkgTrackInst.cmd_support import *
import os, sys
import tempfile

options = None

def parse_options():
    from optparse import OptionParser
    usage = "usage: %prog [options] [make arguments]"
    parser = OptionParser(usage=usage)
    parser.add_option('-v', '--version', dest='show_version',
                      default=False, action='store_true',
                      help='show version and exit')
    parser.add_option('-c', '--check', dest='check_mode',
                      default=False, action='store_true',
                      help="don't generate new plist, but just watch what "
                           "current plist is missing")
    parser.add_option('-s', '--no-substitute', dest='substitute',
                      default=True, action='store_false',
                      help="don't try any substitutions")
    parser.add_option('-o', '--output', dest='output', default='pkg-plist',
                      metavar='FILE', help='filename to write plist output')
    opts, args = parser.parse_args()
    opts.makeargs = list(args)
    return opts

def run_install():
    for i, arg in enumerate(options.makeargs):
        for esc in r'\$`"':
            arg = arg.replace(esc, '\\' + esc)
        options.makeargs[i] = '"' + arg + '"'
    arguments = ' '.join(options.makeargs)

    echo_msg('Building port')
    status = os.system('make build ' + arguments)
    if status:
        errx("`make build' is failed with status code %d." % status)

    echo_msg('Installing port')
    ilog = tempfile.NamedTemporaryFile()
    envs = 'LD_PRELOAD=%s INSTHOOK_LOGFILE=%s' % (INSTHOOK_PATH, ilog.name)
    status = os.system(envs + ' make install ' + arguments)
    if status:
        errx("`make install' is failed with status code %d." % status)
    return open(ilog.name)

def parse_log(logf):
    from PkgTrackInst.InstallSession import InstallSession
    inst = InstallSession()
    for line in logf:
        inst.feed(line[:-1])

def main():
    global options

    import PkgTrackInst
    PkgTrackInst.options = options = parse_options()

    if options.show_version:
        from PkgTrackInst.cmd_support import __banner__
        print __banner__ % 'pkg_genplist'
        raise SystemExit

    # Prepare writer (to exit on error early)
    if options.check_mode:
        from PkgTrackInst.PlistWriter import PlistChecker as workercls
    else:
        from PkgTrackInst.PlistWriter import PlistWriter as workercls
    writer = workercls()

    # Run install command with libinstallwatch preloaded.
    logf = run_install()

    # Parse install log and build directory architecture.
    parse_log(logf)

    # Delete known temporary/build directories from the track list.
    from PkgTrackInst.Trackers import TrackRoot
    for dpfx in IGNORE_PATHS + [os.path.realpath(os.getcwd())]:
        del TrackRoot[dpfx]

    have_entry = []
    def found_entry(ent):
        if ent.created:
            have_entry.append(True)
    TrackRoot.walk(found_entry)
    if not have_entry:
        errx('No file is installed.')

    # Mark mtree directories as not created by the problem.
    from PkgTrackInst.Trackers import MTreeMarker
    MTreeMarker(writer.prefix, MTREE_LOCAL).mark()

    # Generate pkg-plist
    writer.write()

if __name__ == '__main__':
    main()
